package org.q.app.clipboard;

import java.awt.datatransfer.DataFlavor;
import java.io.File;
import java.util.List;

import org.q.toolkit.io.SystemClipboard;

/**
 * Class responsible for writing and reading file operations to the system clipboard.
 * 
 * @author Erik Lievaart
 */
public class ClipboardManager {

	/**
	 * Copy the specified URL's to the clipboard.
	 */
	public void copyUrls(final List<String> urls) {
		SystemClipboard.writeTransferable(new UrlTransferable(new ClipboardTask("copy", urls)));
	}

	/**
	 * Cut the specified URL's to the clipboard.
	 */
	public void cutUrls(final List<String> urls) {
		SystemClipboard.writeTransferable(new UrlTransferable(new ClipboardTask("move", urls)));
	}

	/**
	 * Get the task currently on the clipboard.
	 */
	public ClipboardTask getClipboardTask() {
		ClipboardTask task = SystemClipboard.readFlavor(ClipboardTask.CLIPBOARD_TASK_FLAVOR);
		if (task != null) {
			return task;
		}
		List<File> files = SystemClipboard.readFlavor(DataFlavor.javaFileListFlavor);
		if (files != null) {
			return new ClipboardTask(files);
		}
		return new ClipboardTask("copy", SystemClipboard.readString());
	}
}
