package org.q.app.engine.context;

import java.util.List;

import org.q.app.vfs.VirtualFile;
import org.q.toolkit.lang.collection.ListTool;
import org.q.toolkit.lang.concurrent.Immutable;

/**
 * An immutable snapshot containing the open directory and the selected files for a file list.
 * 
 * @author Erik Lievaart
 */
@Immutable
public class FileSnapshot {
	private final VirtualFile dir;
	private final VirtualFile file;
	private final List<VirtualFile> files;

	/**
	 * Constructor.
	 * 
	 * @param dir
	 *            Directory open in the file list.
	 * @param file
	 *            Primary selection.
	 * @param files
	 *            All selected files.
	 */
	public FileSnapshot(final VirtualFile dir, final VirtualFile file, final List<VirtualFile> files) {
		this.dir = dir;
		this.file = file;
		this.files = ListTool.unmodifiableCopy(files);
	}

	/**
	 * Get the location opened at the time of the snapshot.
	 * 
	 * @return the VirtualFile.
	 */
	public VirtualFile getDir() {
		return dir;
	}

	/**
	 * Get the primary selection at the time of the snapshot.
	 * 
	 * @return the VirtualFile.
	 */
	public VirtualFile getFile() {
		return file;
	}

	/**
	 * Get all selected files at the time of the snapshot.
	 * 
	 * @return the VirtualFiles.
	 */
	public List<VirtualFile> getFiles() {
		return files;
	}

}
