package org.q.app.engine.impl;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.q.app.contract.ValidationException;
import org.q.app.engine.context.BookmarkStore;
import org.q.app.engine.parse.ShellArgument;
import org.q.app.engine.parse.ShellCommand;
import org.q.toolkit.lang.collection.ListTool;
import org.q.toolkit.lang.collection.NewCollection;

import com.google.inject.Inject;

// TODO: variables
public class VariableExaminer {
	private static final List<String> BASE_NAMES = ListTool.of("", "parent", "dir", "url", "urls", "name", "names");
	private static final Set<String> DEFAULT_VARIABLES = listVariableNames();

	@Inject
	private BookmarkStore bookmarks;

	private static Set<String> listVariableNames() {
		List<String> suffixes = ListTool.of("", "~", "1", "2");

		Set<String> result = NewCollection.set();
		for (String prefix : BASE_NAMES) {
			for (String suffix : suffixes) {
				result.add(prefix + suffix);
			}
		}
		return result;
	}

	public void validate(final ShellCommand parsed) throws ValidationException {
		Iterator<ShellArgument> iter = parsed.getArguments();
		while (iter.hasNext()) {
			checkArgument(iter.next());
		}
	}

	private void checkArgument(final ShellArgument argument) throws ValidationException {
		switch (argument.getType()) {

		case BOOKMARK:
			checkBookmark(argument.getValue());
			break;

		case VARIABLE:
			checkVariable(argument.getValue());
			break;
		}
	}

	private void checkBookmark(final String name) throws ValidationException {
		Set<String> names = bookmarks.getVariables().keySet();
		if (!names.contains(name)) {
			throw new ValidationException("Missing Bookmark @" + name + " => " + getKeysStartingWith(name, names));
		}
	}

	private void checkVariable(final String name) throws ValidationException {
		if (!DEFAULT_VARIABLES.contains(name)) {
			throw new ValidationException("Missing Variable $" + name + " => " + getKeysStartingWith(name, BASE_NAMES));
		}
	}

	List<String> getKeysStartingWith(final String prefix, final Collection<String> keys) {
		List<String> result = NewCollection.list();

		for (String key : keys) {
			if (key.startsWith(prefix)) {
				result.add(key);
			}
		}
		return result;
	}

}