package org.q.app.engine.meta;

import java.util.Iterator;
import java.util.List;

import org.q.app.engine.parse.ShellArgument;
import org.q.app.general.ObjectOverrides;
import org.q.toolkit.lang.Check;
import org.q.toolkit.lang.collection.ListTool;
import org.q.toolkit.lang.concurrent.Immutable;

/**
 * The meta data of a single flag of a plugin. This class is immutable.
 * 
 * @author Erik Lievaart
 */
@Immutable
public class FlagMetadata extends ObjectOverrides {

	private final Character flag;
	private final String name;
	private final int arguments;
	private final List<ArgumentMetadata> values;

	FlagMetadata(final String name, final List<ArgumentMetadata> values) {

		Check.notBlank(name);

		this.name = name;
		this.flag = name.charAt(0);
		this.arguments = values.size();
		this.values = ListTool.unmodifiableCopy(values);
	}

	/**
	 * Get the full name of the flag. This is only relevant for documentation/feedback purposes.
	 */
	public String getName() {
		return name;
	}

	Character getFlag() {
		return flag;
	}

	/**
	 * Return the required amount of default {@link ShellArgument}'s optionally overriding them.
	 * 
	 * @param override
	 *            Explicit arguments.
	 * @return For every argument of this flag take one from the iterator. If the Iterator is depleted, use the default
	 *         argument instead.
	 */
	public ShellArgument[] getShellArguments(final Iterator<ShellArgument> override) {
		ShellArgument[] array = new ShellArgument[values.size()];
		for (int i = 0; i < arguments; i++) {
			ShellArgument next = override.hasNext() ? override.next() : ShellArgument.DEFAULT;
			array[i] = next.isDefault() ? values.get(i).getValue() : next;
		}
		return array;
	}
}
