package org.q.app.engine.meta;

import org.q.app.contract.ThreadPolicy;
import org.q.toolkit.lang.Contract;

/**
 * Meta data on how to invoke a plugin.
 * 
 * @author Erik Lievaart
 */
@Contract
public class InvocationMetadata {

	private final boolean swallows;
	private final boolean runnable;
	private final boolean validate;
	private final ThreadPolicy policy;

	InvocationMetadata(final ThreadPolicy tp, final boolean runnable, final boolean swallows, final boolean validate) {
		this.policy = tp;
		this.swallows = swallows;
		this.runnable = runnable;
		this.validate = validate;
	}

	/**
	 * Returns whether or not this command can swallow content.
	 */
	public boolean getSwallows() {
		return swallows;
	}

	/**
	 * Returns whether or not this command can be invoked without any swallowed content.
	 */
	public boolean isInvocable() {
		return runnable;
	}

	boolean isValidateSwallowed() {
		return validate;
	}

	public ThreadPolicy getThreadPolicy() {
		return policy;
	}

}
