package org.q.app.engine.script;

import java.util.List;

import org.apache.commons.lang3.builder.ToStringBuilder;
import org.q.toolkit.lang.collection.FromCollection;

/**
 * This class specifies a list of script files and or raw script code to execute. Script resources are identified by
 * their base name. This is the file name without extension.
 * 
 * @author Erik Lievaart
 */
public class ScriptInvocation {

	private String code = null;
	private final List<String> resources;

	/**
	 * Constructs an empty ScriptInvocation.
	 */
	public ScriptInvocation() {
		this(new String[] {});
	}

	/**
	 * Constructs a ScriptInvocation with a listing of resources to execute.
	 * 
	 * @param resources
	 *            resource names referring to script files.
	 */
	public ScriptInvocation(final String... resources) {
		this.resources = FromCollection.toList(resources);
	}

	/**
	 * Add an additional resource.
	 * 
	 * @param resource
	 *            resource name referring to a script file.
	 */
	public void addResource(final String resource) {
		resources.add(resource);
	}

	List<String> getResources() {
		return resources;
	}

	String getCode() {
		return code;
	}

	/**
	 * Set the raw script code to execute.
	 * 
	 * @param raw
	 *            Raw script code.
	 */
	public void setCode(final String raw) {
		this.code = raw;
	}

	@Override
	public String toString() {
		return new ToStringBuilder(this).append(resources).append(code).toString();
	}

}
