package org.q.app.engine.script;

import com.google.inject.ImplementedBy;

/**
 * Invokes scripts in the Swing event thread.
 * 
 * @author Erik Lievaart
 */
@ImplementedBy(SwingScriptInvoker.class)
public interface SwingScript {

	/**
	 * Invoke the supplied invocations.
	 * 
	 * @param <E>
	 *            Automatically cast the return type to the assigned type.
	 * @param invocation
	 *            supplies raw code and script resources to execute.
	 * @return the result of the script (rarely used).
	 */
	public <E> E invoke(ScriptInvocation invocation);

	/**
	 * Invoke the supplied invocations.
	 * 
	 * @param <E>
	 *            Automatically cast the return type to the assigned type.
	 * @param invocation
	 *            supplies raw code and script resources to execute.
	 * @param config
	 *            Custom configuration of the script environment.
	 * @return the result of the script (rarely used).
	 */
	public <E> E invoke(ScriptConfig config, ScriptInvocation invocation);
}
