package org.q.app.engine.script;

import java.util.concurrent.atomic.AtomicReference;

import org.q.toolkit.swing.SwingThread;

import com.google.inject.Inject;

@SuppressWarnings("unchecked")
class SwingScriptInvoker implements SwingScript {

	@Inject
	private ScriptInvoker runner;

	@Override
	public <E> E invoke(final ScriptInvocation invocation) {
		return (E) invoke(new ScriptConfig(), invocation);
	}

	@Override
	public <E> E invoke(final ScriptConfig config, final ScriptInvocation invocation) {
		final AtomicReference<E> reference = new AtomicReference<E>();

		SwingThread.invokeAndWaitUnchecked(new Runnable() {
			@Override
			public synchronized void run() {
				runner.createEngine(config);
				for (String file : invocation.getResources()) {
					runner.runResource(file);
				}
				reference.set((E) (invocation.getCode() == null ? null : runner.eval(invocation.getCode())));
			}
		});
		return reference.get();
	}

}
