package org.q.app.ui.bind;

import java.awt.Component;
import java.util.Collections;
import java.util.Map;

import javax.swing.JMenuBar;

import org.q.app.config.ConfigurableResources;
import org.q.app.config.ResourcePaths;
import org.q.toolkit.lang.LogTemplate;
import org.q.toolkit.lang.collection.NewCollection;
import org.q.toolkit.reflect.LiteralTool;

import com.google.inject.Inject;
import com.google.inject.Singleton;

/**
 * Map containing all the UI components registered in components.ini. These components will have their accelerators,
 * listeners and key bindings assigned when they are retrieved from the ComponentMap.
 * 
 * @author Erik Lievaart
 */
@Singleton
public class ComponentMap {

	private final Map<String, Object> components = NewCollection.concurrentMap();
	private final LogTemplate log = new LogTemplate(getClass());

	@Inject
	ComponentMap(final JMenuBar menu, final ConfigurableResources resources) {
		Map<String, String> ini = resources.loadIniFile(ResourcePaths.COMPONENTS);
		for (String name : ini.keySet()) {
			log.trace("Registering name % as component %", name, ini.get(name));
			components.put(name, LiteralTool.newInstance(ini.get(name)));
		}
		components.put("q.main.menu", menu);
	}

	@Inject
	void registerBindings(final Bindings bindings) {
		for (String id : components.keySet()) {
			if (components.get(id) instanceof Component) {
				Component component = (Component) components.get(id);
				component.setName(id);
				bindings.configure(id, component);
			}
		}
	}

	/**
	 * Returns an unmodifiable Map containing the components.
	 * 
	 * @return The name of the component as key and the component itself as value.
	 */
	public Map<String, ?> getMap() {
		return Collections.unmodifiableMap(components);
	}
}
