package org.q.app.ui.context;

import java.util.Map;
import java.util.concurrent.atomic.AtomicReference;

import org.q.app.config.ResourcePaths;
import org.q.app.engine.context.Active;
import org.q.app.engine.context.EngineSnapshot;
import org.q.app.engine.script.QObject;
import org.q.app.engine.script.ScriptConfig;
import org.q.app.engine.script.ScriptInvoker;
import org.q.app.vfs.VirtualFile;
import org.q.toolkit.lang.collection.NewCollection;
import org.q.toolkit.swing.SwingThread;

import com.google.inject.Inject;

class EngineSnapshotScript {

	@Inject
	private ScriptInvoker runner;
	@Inject
	private QObject qObject;

	public EngineSnapshot invoke(final Active active, final VirtualFile leftDir, final VirtualFile rightDir) {
		return createEngineContextInSwingThread(new ScriptConfig(createProperties(active, leftDir, rightDir), false));
	}

	private Map<String, Object> createProperties(final Active active, final VirtualFile leftDir,
			final VirtualFile rightDir) {
		Map<String, Object> properties = NewCollection.map();
		properties.put("active", active);
		properties.put("q", qObject);
		if (leftDir != null) {
			properties.put("leftDir", leftDir);
		}
		if (rightDir != null) {
			properties.put("rightDir", rightDir);
		}
		return properties;
	}

	private EngineSnapshot createEngineContextInSwingThread(final ScriptConfig config) {
		AtomicReference<EngineSnapshot> snapshot = new AtomicReference<EngineSnapshot>();

		SwingThread.invokeAndWaitUnchecked(createSwingRunnable(config, snapshot));
		return snapshot.get();
	}

	private Runnable createSwingRunnable(final ScriptConfig config, final AtomicReference<EngineSnapshot> snapshot) {
		return new Runnable() {
			@Override
			public void run() {
				runner.createEngine(config);
				runner.runResource(ResourcePaths.ENGINE_CONTEXT_SCRIPT);
				snapshot.set((EngineSnapshot) runner.eval("return createEngineSnapshot();"));
			}
		};
	}
}
