package org.q.app.ui.render.browser;

import java.util.Map;

import javax.swing.Icon;

import org.q.app.ui.render.list.IconFactory;
import org.q.toolkit.lang.concurrent.RaceCondition;

import com.google.inject.Inject;
import com.google.inject.Singleton;

@Singleton
@RaceCondition("Not ThreadSafe! Access only from Swing event Thread.")
class LocalIconFactory implements IconFactory {

	private final Map<String, Icon> cache;
	private final Icon dir;

	@Inject
	LocalIconFactory(final LocalIconLoader loader) {
		dir = loader.getDirIcon();
		cache = loader.getMapping();
	}

	@Override
	public Icon getIcon(final Object object) {
		return getIcon((VirtualFileWrapper) object);
	}

	private Icon getIcon(final VirtualFileWrapper file) {
		if (file.getVirtualFile().isDirectory()) {
			return dir;
		}
		Icon cached = cache.get(file.getVirtualFile().getExtension().toLowerCase());
		return cached != null ? cached : cache.get("bin");
	}

}
