package org.q.plugins.open;

import java.util.List;

import org.q.app.vfs.VirtualFile;
import org.q.toolkit.lang.collection.NewCollection;

import com.google.inject.Singleton;

/**
 * Singleton for the selected M3u file.
 * 
 * @author Erik Lievaart
 */
@Singleton
public class M3uSelection {

	private List<VirtualFile> available = NewCollection.list();
	private List<VirtualFile> missing = NewCollection.list();

	synchronized void setup(final List<VirtualFile> entries) {
		available.clear();
		missing.clear();

		for (VirtualFile file : entries) {
			if (file.exists()) {
				available.add(file);
			} else {
				missing.add(file);
			}
		}
	}

	/**
	 * Get all entries in the m3u file that are available on the file system.
	 */
	public synchronized List<VirtualFile> getEntries() {
		return available;
	}

	/**
	 * Get all entries in the m3u file, not available on the file system.
	 */
	public synchronized List<VirtualFile> getMissing() {
		return missing;
	}

	/**
	 * Are all entries in this m3u file available on the file system?
	 */
	public synchronized boolean isComplete() {
		return missing.isEmpty();
	}
}
