package org.q.plugins.various;

import java.io.File;
import java.util.Hashtable;
import java.util.Map;
import java.util.TreeMap;

import org.q.app.contract.SingleThreadInterruptedException;
import org.q.toolkit.lang.Check;

class DirCalculator {

	private final File dir;

	DirCalculator(final File dir) {
		Check.isTrue(dir.exists(), "Directory does not exist: %", dir);
		this.dir = dir;
	}

	Map<String, Object> createProperties() {
		Map<String, Long> sizes = new TreeMap<String, Long>();

		long total = 0;
		for (File file : dir.listFiles()) {
			long size = calculate(file);
			sizes.put(file.getName(), size);
			total += size;
		}
		Map<String, Object> properties = new Hashtable<String, Object>();
		properties.put("location", dir.getAbsolutePath());
		properties.put("parentSize", total);
		properties.put("childrenSizes", sizes);
		return properties;
	}

	private long calculate(final File file) {
		if (Thread.interrupted()) {
			throw new SingleThreadInterruptedException();
		}
		if (!file.isDirectory()) {
			return file.length();
		}
		long sum = 0;
		for (File child : file.listFiles()) {
			sum += calculate(child);
		}
		return sum;
	}

}
