package org.q.toolkit.apps.refactor;

import java.io.File;
import java.io.IOException;
import java.util.Collection;
import java.util.List;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.filefilter.NameFileFilter;
import org.apache.commons.io.filefilter.NotFileFilter;
import org.apache.commons.io.filefilter.SuffixFileFilter;
import org.apache.commons.lang3.StringUtils;

public class TextSearchAndReplace {

	private final File root;
	private final String extensions;

	public TextSearchAndReplace(File root, String extensions) {
		this.root = root;
		this.extensions = extensions;
	}

	public void search(final String find) throws IOException {
		System.out.println("\t");
		System.out.println("searching: " + find);

		for (File file : getFiles(root, extensions)) {

			List<String> lines = FileUtils.readLines(file);
			for (int i = 0; i < lines.size(); i++) {
				String line = lines.get(i);
				if (line.contains(find)) {
					System.out.println(file + "(" + (i + 1) + "): " + line);
				}
			}
		}
		System.out.println("complete!");
	}

	public void replace(final String find, final String replace) throws IOException {
		System.out.println("\t");
		System.out.println("replacing : " + find + " => " + replace);

		for (Object file : getFiles(root, extensions)) {
			String contents = FileUtils.readFileToString((File) file);
			String replaced = StringUtils.replace(contents, find, replace);

			if (!replaced.equals(contents)) {
				FileUtils.writeStringToFile((File) file, replaced);
				System.out.println("File has been modified: " + file);
			}
		}
		System.out.println("complete!");
	}

	private static Collection<File> getFiles(final File root, final String ext) {
		SuffixFileFilter fileFilter = new SuffixFileFilter(splitExtensions(ext));
		NotFileFilter dirFilter = new NotFileFilter(new NameFileFilter(".hg"));
		return FileUtils.listFiles(root, fileFilter, dirFilter);
	}

	private static String[] splitExtensions(final String ext) {
		return ext.split("\\s");
	}

}
