package org.q.toolkit.reflect.annotations;

import java.lang.annotation.Annotation;
import java.lang.reflect.Member;

import org.q.toolkit.lang.Check;
import org.q.toolkit.lang.concurrent.Immutable;

@Immutable
abstract class AnnotatedMember<M extends Member, A extends Annotation> {

	private final A annotation;
	private final M member;

	AnnotatedMember(final M member, final A annotation) {
		Check.notNull(member);
		Check.notNull(annotation);

		this.member = member;
		this.annotation = annotation;
	}

	/**
	 * Get the Annotation on the member.
	 */
	public A getAnnotation() {
		return annotation;
	}

	/**
	 * Get the Member that is annotated.
	 */
	public M getMember() {
		return member;
	}

	/**
	 * Get the name of the member.
	 */
	public String getName() {
		return member.getName();
	}

	/**
	 * Get the Class containing the member.
	 */
	public Class<?> getDeclaringClass() {
		return getMember().getDeclaringClass();
	}

	@Override
	public String toString() {
		return String.format("[%s]{ %s }", this.getClass().getSimpleName(), getName());
	}
}
