package org.q.toolkit.swing.menu;

import java.util.List;

import org.q.toolkit.lang.AssertionException;
import org.q.toolkit.lang.Check;
import org.q.toolkit.lang.collection.NewCollection;
import org.q.toolkit.lang.collection.NullPolicy;
import org.q.toolkit.reflect.ReflectException;
import org.q.toolkit.reflect.annotations.AnnotatedMethod;
import org.q.toolkit.reflect.annotations.AnnotationTool;
import org.q.toolkit.reflect.method.MethodWrapper;

public class ReflectionActionBuilder {

	private ReflectionActionBuilder() {
	}

	public static List<NamedAction> createActions(Object object) {
		Check.notNull(object);
		Class<?> literal = object.getClass();
		List<NamedAction> result = NewCollection.list(NullPolicy.REJECT);

		List<AnnotatedMethod<Command>> commands = AnnotationTool.getMethodsAnnotatedWith(literal, Command.class);
		if (commands.isEmpty()) {
			throw new ReflectException("No methods with @Command for " + literal);
		}

		for (AnnotatedMethod<Command> method : commands) {
			MethodWrapper wrapper = method.getMethodWrapper(object);
			Check.isEqual(wrapper.getArgumentTypes().length, 0);
			result.add(new NamedAction(method.getAnnotation().name(), method.getMethodWrapper(object)));
		}
		return result;
	}

	public static NamedAction createAction(Object object, String id) {
		for (NamedAction action : createActions(object)) {
			if (action.getName().equalsIgnoreCase(id)) {
				return action;
			}
		}
		throw new AssertionException("Action not found %", id);
	}

}
